/*1:*/
#line 19 "./mpmathdouble.w"

#include <w2c/config.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include "mplib.h"
#include "mpmp.h" 
#include "mpmathdouble.h" 
#define PI 3.1415926535897932384626433832795028841971
#define fraction_multiplier 4096.0
#define angle_multiplier 16.0 \

#define coef_bound ((7.0/3.0) *fraction_multiplier) 
#define fraction_threshold 0.04096
#define half_fraction_threshold (fraction_threshold/2) 
#define scaled_threshold 0.000122
#define half_scaled_threshold (scaled_threshold/2) 
#define near_zero_angle (0.0256*angle_multiplier) 
#define p_over_v_threshold 0x80000
#define equation_threshold 0.001
#define tfm_warn_threshold 0.0625
#define warning_limit 4.5E+15
#define epsilon 4.5E-15 \

#define unity 1.0
#define two 2.0
#define three 3.0
#define half_unit 0.5
#define three_quarter_unit 0.75 \

#define EL_GORDO (DBL_MAX/2.0-1.0) 
#define one_third_EL_GORDO (EL_GORDO/3.0)  \

#define halfp(A) (integer) ((unsigned) (A) >>1)  \

#define set_cur_cmd(A) mp->cur_mod_->type= (A) 
#define set_cur_mod(A) mp->cur_mod_->data.n->data.dval= (A)  \

#define fraction_half (0.5*fraction_multiplier) 
#define fraction_one (1.0*fraction_multiplier) 
#define fraction_two (2.0*fraction_multiplier) 
#define fraction_three (3.0*fraction_multiplier) 
#define fraction_four (4.0*fraction_multiplier)  \

#define two_to_the(A) (1<<(unsigned) (A) )  \

#define one_eighty_deg (180.0*angle_multiplier) 
#define three_sixty_deg (360.0*angle_multiplier)  \

#define odd(A) ((A) %2==1)  \


#line 27 "./mpmathdouble.w"


/*:1*//*2:*/
#line 29 "./mpmathdouble.w"

/*5:*/
#line 46 "./mpmathdouble.w"

static void mp_double_scan_fractional_token(MP mp,int n);
static void mp_double_scan_numeric_token(MP mp,int n);
static void mp_ab_vs_cd(MP mp,mp_number ret,mp_number a,mp_number b,mp_number c,mp_number d);
static void mp_number_modulo(mp_number a,mp_number b);
static void mp_double_print_number(MP mp,mp_number n);
static char*mp_double_number_tostring(MP mp,mp_number n);
static void mp_double_slow_add(MP mp,mp_number ret,mp_number x_orig,mp_number y_orig);
static void mp_double_square_rt(MP mp,mp_number ret,mp_number x_orig);
static void mp_double_sin_cos(MP mp,mp_number z_orig,mp_number n_cos,mp_number n_sin);
static void mp_init_randoms(MP mp,int seed);
static void mp_number_angle_to_scaled(mp_number A);
static void mp_number_fraction_to_scaled(mp_number A);
static void mp_number_scaled_to_fraction(mp_number A);
static void mp_number_scaled_to_angle(mp_number A);
static void mp_double_m_exp(MP mp,mp_number ret,mp_number x_orig);
static void mp_double_m_log(MP mp,mp_number ret,mp_number x_orig);
static void mp_double_pyth_sub(MP mp,mp_number r,mp_number a,mp_number b);
static void mp_double_pyth_add(MP mp,mp_number r,mp_number a,mp_number b);
static void mp_double_n_arg(MP mp,mp_number ret,mp_number x,mp_number y);
static void mp_double_velocity(MP mp,mp_number ret,mp_number st,mp_number ct,mp_number sf,mp_number cf,mp_number t);
static void mp_set_double_from_int(mp_number A,int B);
static void mp_set_double_from_boolean(mp_number A,int B);
static void mp_set_double_from_scaled(mp_number A,int B);
static void mp_set_double_from_addition(mp_number A,mp_number B,mp_number C);
static void mp_set_double_from_substraction(mp_number A,mp_number B,mp_number C);
static void mp_set_double_from_div(mp_number A,mp_number B,mp_number C);
static void mp_set_double_from_mul(mp_number A,mp_number B,mp_number C);
static void mp_set_double_from_int_div(mp_number A,mp_number B,int C);
static void mp_set_double_from_int_mul(mp_number A,mp_number B,int C);
static void mp_set_double_from_of_the_way(MP mp,mp_number A,mp_number t,mp_number B,mp_number C);
static void mp_number_negate(mp_number A);
static void mp_number_add(mp_number A,mp_number B);
static void mp_number_substract(mp_number A,mp_number B);
static void mp_number_half(mp_number A);
static void mp_number_halfp(mp_number A);
static void mp_number_double(mp_number A);
static void mp_number_add_scaled(mp_number A,int B);
static void mp_number_multiply_int(mp_number A,int B);
static void mp_number_divide_int(mp_number A,int B);
static void mp_double_abs(mp_number A);
static void mp_number_clone(mp_number A,mp_number B);
static void mp_number_swap(mp_number A,mp_number B);
static int mp_round_unscaled(mp_number x_orig);
static int mp_number_to_int(mp_number A);
static int mp_number_to_scaled(mp_number A);
static int mp_number_to_boolean(mp_number A);
static double mp_number_to_double(mp_number A);
static int mp_number_odd(mp_number A);
static int mp_number_equal(mp_number A,mp_number B);
static int mp_number_greater(mp_number A,mp_number B);
static int mp_number_less(mp_number A,mp_number B);
static int mp_number_nonequalabs(mp_number A,mp_number B);
static void mp_number_floor(mp_number i);
static void mp_double_fraction_to_round_scaled(mp_number x);
static void mp_double_number_make_scaled(MP mp,mp_number r,mp_number p,mp_number q);
static void mp_double_number_make_fraction(MP mp,mp_number r,mp_number p,mp_number q);
static void mp_double_number_take_fraction(MP mp,mp_number r,mp_number p,mp_number q);
static void mp_double_number_take_scaled(MP mp,mp_number r,mp_number p,mp_number q);
static mp_number mp_new_number(MP mp,mp_number_type t);
static void mp_free_number(MP mp,mp_number n);
static void mp_set_double_from_double(mp_number A,double B);
static void mp_free_double_math(MP mp);

/*:5*//*20:*/
#line 590 "./mpmathdouble.w"

double mp_double_make_fraction(MP mp,double p,double q);

/*:20*//*22:*/
#line 611 "./mpmathdouble.w"

double mp_double_take_fraction(MP mp,double p,double q);

/*:22*//*25:*/
#line 644 "./mpmathdouble.w"

double mp_double_make_scaled(MP mp,double p,double q);


/*:25*//*27:*/
#line 658 "./mpmathdouble.w"

static void mp_wrapup_numeric_token(MP mp,unsigned char*start,unsigned char*stop);

/*:27*/
#line 30 "./mpmathdouble.w"
;

/*:2*//*7:*/
#line 129 "./mpmathdouble.w"

void*mp_initialize_double_math(MP mp){
math_data*math= (math_data*)mp_xmalloc(mp,1,sizeof(math_data));
fprintf(stdout,"IEEE double math initialized\n");

math->new= mp_new_number;
math->free= mp_free_number;

math->epsilon_t= mp_new_number(mp,mp_scaled_type);
math->epsilon_t->data.dval= epsilon;
math->inf_t= mp_new_number(mp,mp_scaled_type);
math->inf_t->data.dval= EL_GORDO;
math->warning_limit_t= mp_new_number(mp,mp_scaled_type);
math->warning_limit_t->data.dval= warning_limit;
math->one_third_inf_t= mp_new_number(mp,mp_scaled_type);
math->one_third_inf_t->data.dval= one_third_EL_GORDO;
math->unity_t= mp_new_number(mp,mp_scaled_type);
math->unity_t->data.dval= unity;
math->two_t= mp_new_number(mp,mp_scaled_type);
math->two_t->data.dval= two;
math->three_t= mp_new_number(mp,mp_scaled_type);
math->three_t->data.dval= three;
math->half_unit_t= mp_new_number(mp,mp_scaled_type);
math->half_unit_t->data.dval= half_unit;
math->three_quarter_unit_t= mp_new_number(mp,mp_scaled_type);
math->three_quarter_unit_t->data.dval= three_quarter_unit;
math->zero_t= mp_new_number(mp,mp_scaled_type);

math->arc_tol_k= mp_new_number(mp,mp_fraction_type);
math->arc_tol_k->data.dval= (unity/4096);
math->fraction_one_t= mp_new_number(mp,mp_fraction_type);
math->fraction_one_t->data.dval= fraction_one;
math->fraction_half_t= mp_new_number(mp,mp_fraction_type);
math->fraction_half_t->data.dval= fraction_half;
math->fraction_three_t= mp_new_number(mp,mp_fraction_type);
math->fraction_three_t->data.dval= fraction_three;
math->fraction_four_t= mp_new_number(mp,mp_fraction_type);
math->fraction_four_t->data.dval= fraction_four;

math->three_sixty_deg_t= mp_new_number(mp,mp_angle_type);
math->three_sixty_deg_t->data.dval= three_sixty_deg;
math->one_eighty_deg_t= mp_new_number(mp,mp_angle_type);
math->one_eighty_deg_t->data.dval= one_eighty_deg;

math->one_k= mp_new_number(mp,mp_scaled_type);
math->one_k->data.dval= 1024;
math->sqrt_8_e_k= mp_new_number(mp,mp_scaled_type);
math->sqrt_8_e_k->data.dval= 112429/65536.0;
math->twelve_ln_2_k= mp_new_number(mp,mp_fraction_type);
math->twelve_ln_2_k->data.dval= 139548960/65536.0;
math->coef_bound_k= mp_new_number(mp,mp_fraction_type);
math->coef_bound_k->data.dval= coef_bound;
math->coef_bound_minus_1= mp_new_number(mp,mp_fraction_type);
math->coef_bound_minus_1->data.dval= coef_bound-1/65536.0;
math->twelvebits_3= mp_new_number(mp,mp_scaled_type);
math->twelvebits_3->data.dval= 1365/65536.0;
math->twentysixbits_sqrt2_t= mp_new_number(mp,mp_fraction_type);
math->twentysixbits_sqrt2_t->data.dval= 94906266/65536.0;
math->twentyeightbits_d_t= mp_new_number(mp,mp_fraction_type);
math->twentyeightbits_d_t->data.dval= 35596755/65536.0;
math->twentysevenbits_sqrt2_d_t= mp_new_number(mp,mp_fraction_type);
math->twentysevenbits_sqrt2_d_t->data.dval= 25170707/65536.0;

math->fraction_threshold_t= mp_new_number(mp,mp_fraction_type);
math->fraction_threshold_t->data.dval= fraction_threshold;
math->half_fraction_threshold_t= mp_new_number(mp,mp_fraction_type);
math->half_fraction_threshold_t->data.dval= half_fraction_threshold;
math->scaled_threshold_t= mp_new_number(mp,mp_scaled_type);
math->scaled_threshold_t->data.dval= scaled_threshold;
math->half_scaled_threshold_t= mp_new_number(mp,mp_scaled_type);
math->half_scaled_threshold_t->data.dval= half_scaled_threshold;
math->near_zero_angle_t= mp_new_number(mp,mp_angle_type);
math->near_zero_angle_t->data.dval= near_zero_angle;
math->p_over_v_threshold_t= mp_new_number(mp,mp_fraction_type);
math->p_over_v_threshold_t->data.dval= p_over_v_threshold;
math->equation_threshold_t= mp_new_number(mp,mp_scaled_type);
math->equation_threshold_t->data.dval= equation_threshold;
math->tfm_warn_threshold_t= mp_new_number(mp,mp_scaled_type);
math->tfm_warn_threshold_t->data.dval= tfm_warn_threshold;

math->from_int= mp_set_double_from_int;
math->from_boolean= mp_set_double_from_boolean;
math->from_scaled= mp_set_double_from_scaled;
math->from_double= mp_set_double_from_double;
math->from_addition= mp_set_double_from_addition;
math->from_substraction= mp_set_double_from_substraction;
math->from_oftheway= mp_set_double_from_of_the_way;
math->from_div= mp_set_double_from_div;
math->from_mul= mp_set_double_from_mul;
math->from_int_div= mp_set_double_from_int_div;
math->from_int_mul= mp_set_double_from_int_mul;
math->negate= mp_number_negate;
math->add= mp_number_add;
math->substract= mp_number_substract;
math->half= mp_number_half;
math->halfp= mp_number_halfp;
math->do_double= mp_number_double;
math->abs= mp_double_abs;
math->clone= mp_number_clone;
math->swap= mp_number_swap;
math->add_scaled= mp_number_add_scaled;
math->multiply_int= mp_number_multiply_int;
math->divide_int= mp_number_divide_int;
math->to_boolean= mp_number_to_boolean;
math->to_scaled= mp_number_to_scaled;
math->to_double= mp_number_to_double;
math->to_int= mp_number_to_int;
math->odd= mp_number_odd;
math->equal= mp_number_equal;
math->less= mp_number_less;
math->greater= mp_number_greater;
math->nonequalabs= mp_number_nonequalabs;
math->round_unscaled= mp_round_unscaled;
math->floor_scaled= mp_number_floor;
math->fraction_to_round_scaled= mp_double_fraction_to_round_scaled;
math->make_scaled= mp_double_number_make_scaled;
math->make_fraction= mp_double_number_make_fraction;
math->take_fraction= mp_double_number_take_fraction;
math->take_scaled= mp_double_number_take_scaled;
math->velocity= mp_double_velocity;
math->n_arg= mp_double_n_arg;
math->m_log= mp_double_m_log;
math->m_exp= mp_double_m_exp;
math->pyth_add= mp_double_pyth_add;
math->pyth_sub= mp_double_pyth_sub;
math->fraction_to_scaled= mp_number_fraction_to_scaled;
math->scaled_to_fraction= mp_number_scaled_to_fraction;
math->scaled_to_angle= mp_number_scaled_to_angle;
math->angle_to_scaled= mp_number_angle_to_scaled;
math->init_randoms= mp_init_randoms;
math->sin_cos= mp_double_sin_cos;
math->slow_add= mp_double_slow_add;
math->sqrt= mp_double_square_rt;
math->print= mp_double_print_number;
math->tostring= mp_double_number_tostring;
math->modulo= mp_number_modulo;
math->ab_vs_cd= mp_ab_vs_cd;
math->scan_numeric= mp_double_scan_numeric_token;
math->scan_fractional= mp_double_scan_fractional_token;
math->free_math= mp_free_double_math;
return(void*)math;
}

void mp_free_double_math(MP mp){
free_number(((math_data*)mp->math)->three_sixty_deg_t);
free_number(((math_data*)mp->math)->one_eighty_deg_t);
free_number(((math_data*)mp->math)->fraction_one_t);
free_number(((math_data*)mp->math)->zero_t);
free_number(((math_data*)mp->math)->half_unit_t);
free_number(((math_data*)mp->math)->three_quarter_unit_t);
free_number(((math_data*)mp->math)->unity_t);
free_number(((math_data*)mp->math)->two_t);
free_number(((math_data*)mp->math)->three_t);
free_number(((math_data*)mp->math)->one_third_inf_t);
free_number(((math_data*)mp->math)->inf_t);
free_number(((math_data*)mp->math)->warning_limit_t);
free_number(((math_data*)mp->math)->one_k);
free_number(((math_data*)mp->math)->sqrt_8_e_k);
free_number(((math_data*)mp->math)->twelve_ln_2_k);
free_number(((math_data*)mp->math)->coef_bound_k);
free_number(((math_data*)mp->math)->coef_bound_minus_1);
free_number(((math_data*)mp->math)->fraction_threshold_t);
free_number(((math_data*)mp->math)->half_fraction_threshold_t);
free_number(((math_data*)mp->math)->scaled_threshold_t);
free_number(((math_data*)mp->math)->half_scaled_threshold_t);
free_number(((math_data*)mp->math)->near_zero_angle_t);
free_number(((math_data*)mp->math)->p_over_v_threshold_t);
free_number(((math_data*)mp->math)->equation_threshold_t);
free_number(((math_data*)mp->math)->tfm_warn_threshold_t);
free(mp->math);
}

/*:7*//*9:*/
#line 303 "./mpmathdouble.w"

mp_number mp_new_number(MP mp,mp_number_type t){
mp_number n= (mp_number)mp_xmalloc(mp,1,sizeof(struct mp_number_data));
n->data.val= 0;
n->data.dval= 0.0;
n->type= t;
return n;
}

/*:9*//*10:*/
#line 314 "./mpmathdouble.w"

void mp_free_number(MP mp,mp_number n){
(void)mp;
free(n);
}

/*:10*//*11:*/
#line 321 "./mpmathdouble.w"

#ifdef MSVC
#  include <math.h> 
double round(double x)
{
double c,f,d1,d2;

c= ceil(x);
f= floor(x);
d1= fabs(c-x);
d2= fabs(x-f);
if(d1> d2)
return f;
else
return c;
}
#endif
/*:11*//*12:*/
#line 340 "./mpmathdouble.w"

void mp_set_double_from_int(mp_number A,int B){
A->data.dval= B;
}
void mp_set_double_from_boolean(mp_number A,int B){
A->data.dval= B;
}
void mp_set_double_from_scaled(mp_number A,int B){
A->data.dval= B/65536.0;
}
void mp_set_double_from_double(mp_number A,double B){
A->data.dval= (double)(B);
}
void mp_set_double_from_addition(mp_number A,mp_number B,mp_number C){
A->data.dval= B->data.dval+C->data.dval;
}
void mp_set_double_from_substraction(mp_number A,mp_number B,mp_number C){
A->data.dval= B->data.dval-C->data.dval;
}
void mp_set_double_from_div(mp_number A,mp_number B,mp_number C){
A->data.dval= B->data.dval/C->data.dval;
}
void mp_set_double_from_mul(mp_number A,mp_number B,mp_number C){
A->data.dval= B->data.dval*C->data.dval;
}
void mp_set_double_from_int_div(mp_number A,mp_number B,int C){
A->data.dval= B->data.dval/C;
}
void mp_set_double_from_int_mul(mp_number A,mp_number B,int C){
A->data.dval= B->data.dval*C;
}
void mp_set_double_from_of_the_way(MP mp,mp_number A,mp_number t,mp_number B,mp_number C){
A->data.dval= B->data.dval-mp_double_take_fraction(mp,(B->data.dval-C->data.dval),t->data.dval);
}
void mp_number_negate(mp_number A){
A->data.dval= -A->data.dval;
if(A->data.dval==-0.0)
A->data.dval= 0.0;
}
void mp_number_add(mp_number A,mp_number B){
A->data.dval= A->data.dval+B->data.dval;
}
void mp_number_substract(mp_number A,mp_number B){
A->data.dval= A->data.dval-B->data.dval;
}
void mp_number_half(mp_number A){
A->data.dval= A->data.dval/2.0;
}
void mp_number_halfp(mp_number A){
A->data.dval= (A->data.dval/2.0);
}
void mp_number_double(mp_number A){
A->data.dval= A->data.dval*2.0;
}
void mp_number_add_scaled(mp_number A,int B){
A->data.dval= A->data.dval+(B/65536.0);
}
void mp_number_multiply_int(mp_number A,int B){
A->data.dval= (double)(A->data.dval*B);
}
void mp_number_divide_int(mp_number A,int B){
A->data.dval= A->data.dval/(double)B;
}
void mp_double_abs(mp_number A){
A->data.dval= fabs(A->data.dval);
}
void mp_number_clone(mp_number A,mp_number B){
A->data.dval= B->data.dval;
}
void mp_number_swap(mp_number A,mp_number B){
double swap_tmp= A->data.dval;
A->data.dval= B->data.dval;
B->data.dval= swap_tmp;
}
void mp_number_fraction_to_scaled(mp_number A){
A->type= mp_scaled_type;
A->data.dval= A->data.dval/fraction_multiplier;
}
void mp_number_angle_to_scaled(mp_number A){
A->type= mp_scaled_type;
if(A->data.dval>=0){
A->data.dval= round(A->data.dval)/angle_multiplier;
}else{
A->data.dval= -((-round(A->data.dval))/angle_multiplier);
}
}
void mp_number_scaled_to_fraction(mp_number A){
A->type= mp_fraction_type;
A->data.dval= A->data.dval*fraction_multiplier;
}
void mp_number_scaled_to_angle(mp_number A){
A->type= mp_angle_type;
A->data.dval= A->data.dval*angle_multiplier;
}


/*:12*//*13:*/
#line 438 "./mpmathdouble.w"

int mp_number_to_scaled(mp_number A){
return(int)round(A->data.dval*65536.0);
}
int mp_number_to_int(mp_number A){
return(int)(A->data.dval);
}
int mp_number_to_boolean(mp_number A){
return(A->data.dval!=0.0);
}
double mp_number_to_double(mp_number A){
return A->data.dval;
}
int mp_number_odd(mp_number A){
return odd((int)round(A->data.dval*65536.0));
}
int mp_number_equal(mp_number A,mp_number B){
return(A->data.dval==B->data.dval);
}
int mp_number_greater(mp_number A,mp_number B){
return(A->data.dval> B->data.dval);
}
int mp_number_less(mp_number A,mp_number B){
return(A->data.dval<B->data.dval);
}
int mp_number_nonequalabs(mp_number A,mp_number B){
return(!(fabs(A->data.dval)==fabs(B->data.dval)));
}

/*:13*//*16:*/
#line 498 "./mpmathdouble.w"

char*mp_double_number_tostring(MP mp,mp_number n){
static char set[64];
int l= 0;
char*try= mp_xmalloc(mp,64,1);
snprintf(set,64,"%32.16g",n->data.dval);
while(set[l]==' ')l++;
strcpy(try,set+l);
return try;
}


/*:16*//*17:*/
#line 510 "./mpmathdouble.w"

void mp_double_print_number(MP mp,mp_number n){
char*str= mp_double_number_tostring(mp,n);
mp_print(mp,str);
free(str);
}




/*:17*//*18:*/
#line 524 "./mpmathdouble.w"

void mp_double_slow_add(MP mp,mp_number ret,mp_number x_orig,mp_number y_orig){
double x,y;
x= x_orig->data.dval;
y= y_orig->data.dval;
if(x>=0){
if(y<=EL_GORDO-x){
ret->data.dval= x+y;
}else{
mp->arith_error= true;
ret->data.dval= EL_GORDO;
}
}else if(-y<=EL_GORDO+x){
ret->data.dval= x+y;
}else{
mp->arith_error= true;
ret->data.dval= -EL_GORDO;
}
}

/*:18*//*19:*/
#line 582 "./mpmathdouble.w"

double mp_double_make_fraction(MP mp,double p,double q){
return((p/q)*fraction_multiplier);
}
void mp_double_number_make_fraction(MP mp,mp_number ret,mp_number p,mp_number q){
ret->data.dval= mp_double_make_fraction(mp,p->data.dval,q->data.dval);
}

/*:19*//*21:*/
#line 603 "./mpmathdouble.w"

double mp_double_take_fraction(MP mp,double p,double q){
return((p*q)/fraction_multiplier);
}
void mp_double_number_take_fraction(MP mp,mp_number ret,mp_number p,mp_number q){
ret->data.dval= mp_double_take_fraction(mp,p->data.dval,q->data.dval);
}

/*:21*//*23:*/
#line 624 "./mpmathdouble.w"

void mp_double_number_take_scaled(MP mp,mp_number ret,mp_number p_orig,mp_number q_orig){
ret->data.dval= p_orig->data.dval*q_orig->data.dval;
}


/*:23*//*24:*/
#line 636 "./mpmathdouble.w"

double mp_double_make_scaled(MP mp,double p,double q){
return p/q;
}
void mp_double_number_make_scaled(MP mp,mp_number ret,mp_number p_orig,mp_number q_orig){
ret->data.dval= p_orig->data.dval/q_orig->data.dval;
}

/*:24*//*28:*/
#line 661 "./mpmathdouble.w"

void mp_wrapup_numeric_token(MP mp,unsigned char*start,unsigned char*stop){
double result;
char*end= (char*)stop;
errno= 0;
result= strtod((char*)start,&end);
if(errno==0){
set_cur_mod(result);
if(result>=warning_limit){
if(internal_value(mp_warning_check)->data.dval> 0&&
(mp->scanner_status!=tex_flushing)){
char msg[256];
const char*hlp[]= {"Continue and I'll try to cope",
"with that big value; but it might be dangerous.",
"(Set warningcheck:=0 to suppress this message.)",
NULL};
mp_snprintf(msg,256,"Number is too large (%g)",result);
;
mp_error(mp,msg,hlp,true);
}
}
}else if(mp->scanner_status!=tex_flushing){
const char*hlp[]= {"I could not handle this number specification",
"probably because it is out of range. Error:",
"",
NULL};
hlp[2]= strerror(errno);
mp_error(mp,"Enormous number has been reduced.",hlp,false);
;
set_cur_mod(EL_GORDO);
}
set_cur_cmd(mp_numeric_token);
}

/*:28*//*29:*/
#line 695 "./mpmathdouble.w"

static void find_exponent(MP mp){
if(mp->buffer[mp->cur_input.loc_field]=='e'||
mp->buffer[mp->cur_input.loc_field]=='E'){
mp->cur_input.loc_field++;
if(mp->buffer[mp->cur_input.loc_field]=='+'||
mp->buffer[mp->cur_input.loc_field]=='-'){
mp->cur_input.loc_field++;
}
while(mp->char_class[mp->buffer[mp->cur_input.loc_field]]==digit_class){
mp->cur_input.loc_field++;
}
}
}
void mp_double_scan_fractional_token(MP mp,int n){
unsigned char*start= &mp->buffer[mp->cur_input.loc_field-1];
unsigned char*stop;
while(mp->char_class[mp->buffer[mp->cur_input.loc_field]]==digit_class){
mp->cur_input.loc_field++;
}
find_exponent(mp);
stop= &mp->buffer[mp->cur_input.loc_field-1];
mp_wrapup_numeric_token(mp,start,stop);
}


/*:29*//*30:*/
#line 724 "./mpmathdouble.w"

void mp_double_scan_numeric_token(MP mp,int n){
unsigned char*start= &mp->buffer[mp->cur_input.loc_field-1];
unsigned char*stop;
while(mp->char_class[mp->buffer[mp->cur_input.loc_field]]==digit_class){
mp->cur_input.loc_field++;
}
if(mp->buffer[mp->cur_input.loc_field]=='.'&&
mp->buffer[mp->cur_input.loc_field+1]!='.'){
mp->cur_input.loc_field++;
while(mp->char_class[mp->buffer[mp->cur_input.loc_field]]==digit_class){
mp->cur_input.loc_field++;
}
}
find_exponent(mp);
stop= &mp->buffer[mp->cur_input.loc_field-1];
mp_wrapup_numeric_token(mp,start,stop);
}

/*:30*//*32:*/
#line 777 "./mpmathdouble.w"

void mp_double_velocity(MP mp,mp_number ret,mp_number st,mp_number ct,mp_number sf,
mp_number cf,mp_number t){
double acc,num,denom;
acc= mp_double_take_fraction(mp,st->data.dval-(sf->data.dval/16.0),
sf->data.dval-(st->data.dval/16.0));
acc= mp_double_take_fraction(mp,acc,ct->data.dval-cf->data.dval);
num= fraction_two+mp_double_take_fraction(mp,acc,sqrt(2)*fraction_one);
denom= 
fraction_three+mp_double_take_fraction(mp,ct->data.dval,3*fraction_half*(sqrt(5.0)-1.0))
+mp_double_take_fraction(mp,cf->data.dval,3*fraction_half*(3.0-sqrt(5.0)));
if(t->data.dval!=unity)
num= mp_double_make_scaled(mp,num,t->data.dval);
if(num/4>=denom){
ret->data.dval= fraction_four;
}else{
ret->data.dval= mp_double_make_fraction(mp,num,denom);
}
}


/*:32*//*33:*/
#line 803 "./mpmathdouble.w"

void mp_ab_vs_cd(MP mp,mp_number ret,mp_number a_orig,mp_number b_orig,mp_number c_orig,mp_number d_orig){
integer q,r;
integer a,b,c,d;
(void)mp;
a= a_orig->data.dval;
b= b_orig->data.dval;
c= c_orig->data.dval;
d= d_orig->data.dval;
/*34:*/
#line 838 "./mpmathdouble.w"

if(a<0){
a= -a;
b= -b;
}
if(c<0){
c= -c;
d= -d;
}
if(d<=0){
if(b>=0){
if((a==0||b==0)&&(c==0||d==0))
ret->data.dval= 0;
else
ret->data.dval= 1;
return;
}
if(d==0){
ret->data.dval= (a==0?0:-1);
return;
}
q= a;
a= c;
c= q;
q= -b;
b= -d;
d= q;
}else if(b<=0){
if(b<0&&a> 0){
ret->data.dval= -1;
return;
}
ret->data.dval= (c==0?0:-1);
return;
}

/*:34*/
#line 812 "./mpmathdouble.w"
;
while(1){
q= a/d;
r= c/b;
if(q!=r){
ret->data.dval= (q> r?1:-1);
return;
}
q= a%d;
r= c%b;
if(r==0){
ret->data.dval= (q?1:0);
return;
}
if(q==0){
ret->data.dval= -1;
return;
}
a= b;
b= q;
c= d;
d= r;
}
}


/*:33*//*36:*/
#line 879 "./mpmathdouble.w"

int mp_round_unscaled(mp_number x_orig){
int x= (int)round(x_orig->data.dval);
return x;
}

/*:36*//*37:*/
#line 887 "./mpmathdouble.w"

void mp_number_floor(mp_number i){
i->data.dval= floor(i->data.dval);
}

/*:37*//*38:*/
#line 893 "./mpmathdouble.w"

void mp_double_fraction_to_round_scaled(mp_number x_orig){
double x= x_orig->data.dval;
x_orig->type= mp_scaled_type;
x_orig->data.dval= round(x)/fraction_multiplier;
}



/*:38*//*40:*/
#line 908 "./mpmathdouble.w"

void mp_double_square_rt(MP mp,mp_number ret,mp_number x_orig){
double x;
x= x_orig->data.dval;
if(x<=0){
/*41:*/
#line 920 "./mpmathdouble.w"

{
if(x<0){
char msg[256];
const char*hlp[]= {
"Since I don't take square roots of negative numbers,",
"I'm zeroing this one. Proceed, with fingers crossed.",
NULL};
char*xstr= mp_double_number_tostring(mp,x_orig);
mp_snprintf(msg,256,"Square root of %s has been replaced by 0",xstr);
free(xstr);
;
mp_error(mp,msg,hlp,true);
}
ret->data.dval= 0;
return;
}


/*:41*/
#line 913 "./mpmathdouble.w"
;
}else{
ret->data.dval= sqrt(x);
}
}


/*:40*//*42:*/
#line 941 "./mpmathdouble.w"

void mp_double_pyth_add(MP mp,mp_number ret,mp_number a_orig,mp_number b_orig){
double a,b;
a= fabs(a_orig->data.dval);
b= fabs(b_orig->data.dval);
errno= 0;
ret->data.dval= sqrt(a*a+b*b);
if(errno){
mp->arith_error= true;
ret->data.dval= EL_GORDO;
}
}


/*:42*//*43:*/
#line 957 "./mpmathdouble.w"

void mp_double_pyth_sub(MP mp,mp_number ret,mp_number a_orig,mp_number b_orig){
double a,b;
a= fabs(a_orig->data.dval);
b= fabs(b_orig->data.dval);
if(a<=b){
/*44:*/
#line 971 "./mpmathdouble.w"

{
if(a<b){
char msg[256];
const char*hlp[]= {
"Since I don't take square roots of negative numbers,",
"I'm zeroing this one. Proceed, with fingers crossed.",
NULL};
char*astr= mp_double_number_tostring(mp,a_orig);
char*bstr= mp_double_number_tostring(mp,b_orig);
mp_snprintf(msg,256,"Pythagorean subtraction %s+-+%s has been replaced by 0",astr,bstr);
free(astr);
free(bstr);
;
mp_error(mp,msg,hlp,true);
}
a= 0;
}


/*:44*/
#line 963 "./mpmathdouble.w"
;
}else{
a= sqrt(a*a-b*b);
}
ret->data.dval= a;
}


/*:43*//*46:*/
#line 1000 "./mpmathdouble.w"

void mp_double_m_log(MP mp,mp_number ret,mp_number x_orig){
if(x_orig->data.dval<=0){
/*47:*/
#line 1009 "./mpmathdouble.w"

{
char msg[256];
const char*hlp[]= {
"Since I don't take logs of non-positive numbers,",
"I'm zeroing this one. Proceed, with fingers crossed.",
NULL};
char*xstr= mp_double_number_tostring(mp,x_orig);
mp_snprintf(msg,256,"Logarithm of %s has been replaced by 0",xstr);
free(xstr);
;
mp_error(mp,msg,hlp,true);
ret->data.dval= 0;
}


/*:47*/
#line 1003 "./mpmathdouble.w"
;
}else{
ret->data.dval= log(x_orig->data.dval*256.0);
}
}

/*:46*//*48:*/
#line 1028 "./mpmathdouble.w"

void mp_double_m_exp(MP mp,mp_number ret,mp_number x_orig){
errno= 0;
ret->data.dval= exp(x_orig->data.dval/256.0);
if(errno){
if(x_orig->data.dval> 0){
mp->arith_error= true;
ret->data.dval= EL_GORDO;
}else{
ret->data.dval= 0;
}
}
}


/*:48*//*49:*/
#line 1046 "./mpmathdouble.w"

void mp_double_n_arg(MP mp,mp_number ret,mp_number x_orig,mp_number y_orig){
if(x_orig->data.dval==0.0&&y_orig->data.dval==0.0){
/*50:*/
#line 1059 "./mpmathdouble.w"

{
const char*hlp[]= {
"The `angle' between two identical points is undefined.",
"I'm zeroing this one. Proceed, with fingers crossed.",
NULL};
mp_error(mp,"angle(0,0) is taken as zero",hlp,true);
;
ret->data.dval= 0;
}


/*:50*/
#line 1049 "./mpmathdouble.w"
;
}else{
ret->type= mp_angle_type;
ret->data.dval= atan2(y_orig->data.dval,x_orig->data.dval)*(180.0/PI)*angle_multiplier;
if(ret->data.dval==-0.0)
ret->data.dval= 0.0;
}
}


/*:49*//*53:*/
#line 1089 "./mpmathdouble.w"

void mp_double_sin_cos(MP mp,mp_number z_orig,mp_number n_cos,mp_number n_sin){
double rad;
rad= (z_orig->data.dval/angle_multiplier)*PI/180.0;
n_cos->data.dval= cos(rad)*fraction_multiplier;
n_sin->data.dval= sin(rad)*fraction_multiplier;
}

/*:53*//*54:*/
#line 1099 "./mpmathdouble.w"

void mp_init_randoms(MP mp,int seed){
int j,jj,k;
int i;
j= abs(seed);
while(j>=fraction_one){
j= j/2;
}
k= 1;
for(i= 0;i<=54;i++){
jj= k;
k= j-k;
j= jj;
if(k<0)
k+= fraction_one;
mp->randoms[(i*21)%55]->data.dval= j;
}
mp_new_randoms(mp);
mp_new_randoms(mp);
mp_new_randoms(mp);
}

/*:54*//*55:*/
#line 1121 "./mpmathdouble.w"

static double modulus(double left,double right);
double modulus(double left,double right){
double quota= left/right;
double frac,tmp;
frac= modf(quota,&tmp);

frac*= right;
return frac;
}
void mp_number_modulo(mp_number a,mp_number b){
a->data.dval= modulus(a->data.dval,b->data.dval);
}/*:55*/
